@@ -3,7 +3,7 @@
 
 gwr.write<-function(x,fn="GWRresults")
 {
-   if(class(x) != "gwrm") stop("It's not a gwm object")
+   if(!inherits(x, "gwrm")) stop("It's not a gwm object")
    fn1<-paste(fn,".txt",sep="")
    #fn2<-paste(fn,".csv",sep="")
    sink(fn1)
@@ -15,7 +15,7 @@ gwr.write<-function(x,fn="GWRresults")
 }
 gwr.write.shp<-function(x,fn="GWRresults")
 {
-   if(class(x) != "gwrm") stop("It's not a gwm object")
+   if(!inherits(x, "gwrm")) stop("It's not a gwm object")
    SDF<-x$SDF
    if (is(SDF, "SpatialPointsDataFrame"))
      writePointsShape(SDF,fn=fn, max_nchar= 256)
@@ -27,7 +27,7 @@ gwr.write.shp<-function(x,fn="GWRresults")
 # This version of this function is kept to make the code work with the early versions of GWmodel (before 2.0-1)
 writeGWR<-function(x,fn="GWRresults")
 {
-   if(class(x) != "gwrm") stop("It's not a gwm object")
+   if(!inherits(x, "gwrm")) stop("It's not a gwm object")
    fn1<-paste(fn,".txt",sep="")
    #fn2<-paste(fn,".csv",sep="")
    sink(fn1)
@@ -41,7 +41,7 @@ writeGWR<-function(x,fn="GWRresults")
 # This version of this function is kept to make the code work with the early versions of GWmodel (before 2.0-1)
 writeGWR.shp<-function(x,fn="GWRresults")
 {
-   if(class(x) != "gwrm") stop("It's not a gwm object")
+   if(!inherits(x, "gwrm")) stop("It's not a gwm object")
    SDF<-x$SDF
    if (is(SDF, "SpatialPointsDataFrame"))
      writePointsShape(SDF,fn=fn, max_nchar= 256)
